package dev.fyloz.colorrecipesexplorer.rest

import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeEditRecipes
import dev.fyloz.colorrecipesexplorer.config.annotations.PreAuthorizeViewRecipes
import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.service.MixService
import dev.fyloz.colorrecipesexplorer.service.RecipeImageService
import dev.fyloz.colorrecipesexplorer.service.RecipeService
import org.springframework.context.annotation.Profile
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.*
import org.springframework.web.multipart.MultipartFile
import javax.validation.Valid


private const val RECIPE_CONTROLLER_PATH = "api/recipe"
private const val MIX_CONTROLLER_PATH = "api/recipe/mix"

@RestController
@RequestMapping(RECIPE_CONTROLLER_PATH)
@Profile("!emergency")
@PreAuthorizeViewRecipes
class RecipeController(
    private val recipeService: RecipeService,
    private val recipeImageService: RecipeImageService
) {
    @GetMapping
    fun getAll(@RequestParam(required = false) name: String?) =
        if (name == null)
            ok(recipeService.getAllForOutput())
        else
            ok(with(recipeService) {
                getAllByName(name).map { it.toOutput() }
            })

    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(recipeService.getByIdForOutput(id))

    @PostMapping
    @PreAuthorizeEditRecipes
    fun save(@Valid @RequestBody recipe: RecipeSaveDto) =
        created<RecipeOutputDto>(RECIPE_CONTROLLER_PATH) {
            with(recipeService) {
                save(recipe).toOutput()
            }
        }

    @PutMapping
    @PreAuthorizeEditRecipes
    fun update(@Valid @RequestBody recipe: RecipeUpdateDto) =
        noContent {
            recipeService.update(recipe)
        }

    @PutMapping("public")
    @PreAuthorize("hasAuthority('EDIT_RECIPES_PUBLIC_DATA')")
    fun updatePublicData(@Valid @RequestBody publicDataDto: RecipePublicDataDto) =
        noContent {
            recipeService.updatePublicData(publicDataDto)
        }

    @DeleteMapping("{id}")
    @PreAuthorizeEditRecipes
    fun deleteById(@PathVariable id: Long) =
        noContent {
            recipeService.deleteById(id)
        }

    @PutMapping("{recipeId}/image", consumes = [MediaType.MULTIPART_FORM_DATA_VALUE])
    @PreAuthorizeEditRecipes
    fun downloadImage(@PathVariable recipeId: Long, image: MultipartFile): ResponseEntity<RecipeOutputDto> {
        recipeImageService.download(image, recipeService.getById(recipeId))
        return getById(recipeId)
    }

    @DeleteMapping("{recipeId}/image/{name}")
    @PreAuthorizeEditRecipes
    fun deleteImage(@PathVariable recipeId: Long, @PathVariable name: String) =
        noContent {
            recipeImageService.delete(recipeService.getById(recipeId), name)
        }
}

@RestController
@RequestMapping(MIX_CONTROLLER_PATH)
@Profile("!emergency")
@PreAuthorizeViewRecipes
class MixController(private val mixService: MixService) {
    @GetMapping("{id}")
    fun getById(@PathVariable id: Long) =
        ok(mixService.getByIdForOutput(id))

    @PostMapping
    @PreAuthorizeEditRecipes
    fun save(@Valid @RequestBody mix: MixSaveDto) =
        created<Mix>(MIX_CONTROLLER_PATH) {
            mixService.save(mix)
        }

    @PutMapping
    @PreAuthorizeEditRecipes
    fun update(@Valid @RequestBody mix: MixUpdateDto) =
        noContent {
            mixService.update(mix)
        }

    @DeleteMapping("{id}")
    @PreAuthorizeEditRecipes
    fun deleteById(@PathVariable id: Long) =
        noContent {
            mixService.deleteById(id)
        }
}
